@@ -4,8 +4,9 @@ use crate::{
     formatters::{
         expression::{format_expression, format_var},
         general::{
-            format_contained_span, format_end_token, format_punctuated, format_symbol,
-            format_token_reference, try_format_punctuated, EndTokenType,
+            format_contained_punctuated_multiline, format_contained_span, format_end_token,
+            format_punctuated, format_symbol, format_token_reference, try_format_punctuated,
+            EndTokenType,
         },
         table::{create_table_braces, format_multiline_table, format_singleline_table, TableType},
         trivia::{
@@ -13,19 +14,17 @@ use crate::{
             UpdateLeadingTrivia, UpdateTrailingTrivia,
         },
         trivia_util::{
-            contains_comments, token_trivia_contains_comments, trivia_is_comment,
-            trivia_is_newline, type_info_trailing_trivia,
+            contains_comments, take_type_argument_trailing_comments,
+            token_trivia_contains_comments, trivia_is_comment, trivia_is_newline,
+            type_info_trailing_trivia,
         },
     },
     shape::Shape,
 };
-use full_moon::ast::{
-    punctuated::Pair,
-    types::{
-        CompoundAssignment, CompoundOp, ExportedTypeDeclaration, GenericDeclaration,
-        GenericDeclarationParameter, GenericParameterInfo, IndexedTypeInfo, TypeArgument,
-        TypeAssertion, TypeDeclaration, TypeField, TypeFieldKey, TypeInfo, TypeSpecifier,
-    },
+use full_moon::ast::types::{
+    CompoundAssignment, CompoundOp, ExportedTypeDeclaration, GenericDeclaration,
+    GenericDeclarationParameter, GenericParameterInfo, IndexedTypeInfo, TypeArgument,
+    TypeAssertion, TypeDeclaration, TypeField, TypeFieldKey, TypeInfo, TypeSpecifier,
 };
 use full_moon::ast::{punctuated::Punctuated, span::ContainedSpan};
 use full_moon::tokenizer::{Token, TokenReference, TokenType};
@@ -118,38 +117,14 @@ pub fn format_type_info(ctx: &Context, type_info: &TypeInfo, shape: Shape) -> Ty
                     .over_budget(); // 2 = opening/closing parens, 4 = " -> "
 
             let (parentheses, arguments, shape) = if force_multiline {
-                let start_parens = fmt_symbol!(ctx, start_parens, "(", shape)
-                    .update_trailing_trivia(FormatTriviaType::Append(vec![create_newline_trivia(
-                        ctx,
-                    )]));
-                let end_parens =
-                    format_end_token(ctx, end_parens, EndTokenType::ClosingParens, shape)
-                        .update_leading_trivia(FormatTriviaType::Append(vec![
-                            create_newline_trivia(ctx),
-                            create_indent_trivia(ctx, shape),
-                        ]));
-
-                let parentheses = ContainedSpan::new(start_parens, end_parens);
-                let mut formatted_arguments = Punctuated::new();
-
-                for pair in arguments.pairs() {
-                    // Reset the shape (as the parameter is on a newline), and increment the additional indent level
-                    let shape = shape.reset().increment_additional_indent();
-
-                    let parameter = format_type_argument(ctx, pair.value(), shape)
-                        .update_leading_trivia(FormatTriviaType::Append(vec![
-                            create_indent_trivia(ctx, shape),
-                        ]));
-
-                    let punctuation = pair.punctuation().map(|punctuation| {
-                        fmt_symbol!(ctx, punctuation, ",", shape).update_trailing_trivia(
-                            FormatTriviaType::Append(vec![create_newline_trivia(ctx)]),
-                        )
-                    });
-
-                    formatted_arguments.push(Pair::new(parameter, punctuation))
-                }
-
+                let (parentheses, formatted_arguments) = format_contained_punctuated_multiline(
+                    ctx,
+                    parentheses,
+                    arguments,
+                    format_type_argument,
+                    take_type_argument_trailing_comments,
+                    shape,
+                );
                 let shape = shape.reset() + 1; // 1 = ")"
 
                 (parentheses, formatted_arguments, shape)
@@ -363,14 +338,29 @@ pub fn format_type_info(ctx: &Context, type_info: &TypeInfo, shape: Shape) -> Ty
 
 pub fn hang_type_info(ctx: &Context, type_info: TypeInfo, shape: Shape) -> TypeInfo {
     match type_info {
-        TypeInfo::Union { left, pipe, right } => TypeInfo::Union {
-            left,
-            pipe: pipe.update_leading_trivia(FormatTriviaType::Replace(vec![
-                create_newline_trivia(ctx),
-                create_indent_trivia(ctx, shape),
-            ])),
-            right: Box::new(hang_type_info(ctx, *right, shape)),
-        },
+        TypeInfo::Union { left, pipe, right } => {
+            let mut pipe_leading_trivia: Vec<_> = pipe
+                .leading_trivia()
+                .filter(|token| trivia_is_comment(token))
+                .flat_map(|trivia| {
+                    // Prepend an indent before the comment, and append a newline after the comments
+                    vec![
+                        create_newline_trivia(ctx),
+                        create_indent_trivia(ctx, shape),
+                        trivia.to_owned(),
+                    ]
+                })
+                .collect();
+
+            pipe_leading_trivia.push(create_newline_trivia(ctx));
+            pipe_leading_trivia.push(create_indent_trivia(ctx, shape));
+
+            TypeInfo::Union {
+                left,
+                pipe: pipe.update_leading_trivia(FormatTriviaType::Replace(pipe_leading_trivia)),
+                right: Box::new(hang_type_info(ctx, *right, shape)),
+            }
+        }
         _ => type_info,
     }
 }
@@ -502,6 +492,25 @@ pub fn format_type_assertion(
     TypeAssertion::new(cast_to).with_assertion_op(assertion_op)
 }
 
+fn should_hang_type(type_info: &TypeInfo) -> bool {
+    // Only hang if its a binary type info, since it doesn't matter for unary types
+    match type_info {
+        TypeInfo::Union { left, pipe, right } => {
+            type_info_trailing_trivia(left)
+                .iter()
+                .any(trivia_is_comment)
+                || should_hang_type(left)
+                || contains_comments(pipe)
+                || full_moon::node::Node::surrounding_trivia(right)
+                    .0
+                    .iter()
+                    .any(|trivia| trivia_is_comment(trivia))
+                || should_hang_type(right)
+        }
+        _ => false,
+    }
+}
+
 fn format_type_declaration(
     ctx: &Context,
     type_declaration: &TypeDeclaration,
@@ -549,7 +558,7 @@ fn format_type_declaration(
 
     let shape = shape.take_last_line(&strip_trailing_trivia(&type_definition));
 
-    if shape.over_budget() {
+    if should_hang_type(type_declaration.type_definition()) || shape.over_budget() {
         let shape = shape.increment_additional_indent();
         equal_token = equal_token.update_trailing_trivia(FormatTriviaType::Replace(vec![
             create_newline_trivia(ctx),
